#include <iostream>
#include "avi.h"

using namespace std;

avi::avi(char* filename) {
	name = filename;

	frames = 0;
	height = 0;
	width = 0;
	fps = 30.0; // default

	moviefile = NULL;
	fileoffset = NULL;
}

avi::~avi() {
	frames = 0;
	height = 0;
	width = 0;
	fps = 0.0;
   
	name = NULL;
	delete fileoffset;
	fileoffset = NULL;
	
	fclose(moviefile);
	moviefile = NULL;
}

void avi::read_video() {
	char buffer[4];
	uint temp[1024];
	long size;
	
	if((moviefile = fopen(name,"rb")) == NULL) {
		exit(1);
	}

	fread(buffer,sizeof(char),4,moviefile); // "RIFF"
	if(memcmp(buffer,"RIFF",4) != 0) {
		fclose(moviefile);
		exit(1);
	}
	
	fread(&size,sizeof(int),1,moviefile); // size
	fread(buffer,sizeof(char),4,moviefile); // "AVI "
	fread(buffer,sizeof(char),4,moviefile); // "LIST"

	fread(&size,sizeof(int),1,moviefile); // size (LISTHEADERCHUNCK)
	fread(buffer,sizeof(char),4,moviefile); // "hdrl"

	fread(buffer,sizeof(char),4,moviefile); // "avih"
	fread(&size,sizeof(int),1,moviefile); // size (AVIHEADER)
	size = SWAP_BYTE_ORDER(size); // Convert to Little Endian Architecture
	fread(temp,size,1,moviefile);
	fps  = 1000000/SWAP_BYTE_ORDER(temp[0]); // Convert to Little Endian Architecture
	frames = (uint)(SWAP_BYTE_ORDER(temp[4])); // Convert to Little Endian Architecture
	width  = (uint)(SWAP_BYTE_ORDER(temp[8])); // Convert to Little Endian Architecture
	height = (uint)(SWAP_BYTE_ORDER(temp[9])); // Convert to Little Endian Architecture

	fread(buffer,sizeof(char),4,moviefile); // "LIST"
	fread(&size,sizeof(int),1,moviefile); // size (LISTHEADERCHUNCK2)
	fread(buffer,sizeof(char),4,moviefile); // "strl"
	
	fread(buffer,sizeof(char),4,moviefile); // "strh"
	fread(&size,sizeof(int),1,moviefile); // size (AVISTREAMHEADER)
	size = SWAP_BYTE_ORDER(size); // Convert to Little Endian Architecture
	fread(temp,size,1,moviefile);

	fread(buffer,sizeof(char),4,moviefile); // "strf"
    fread(&size,sizeof(int),1,moviefile); // size (AVISTREAMFORMAT)
	size = SWAP_BYTE_ORDER(size); // Convert to Little Endian Architecture
	fread(temp,size,1,moviefile);

    fread(buffer,sizeof(char),4,moviefile); // "strn" || "JUNK"
	if (memcmp(buffer,"strn",4) == 0) {
		fread(&size,sizeof(int),1,moviefile);
		size = SWAP_BYTE_ORDER(size) + 1; // Convert to Little Endian Architecture
		fseek(moviefile,size,SEEK_CUR);
		fread(buffer,sizeof(char),4,moviefile); // chunck (LISTMOVIECHUNCK)
	}
	if (memcmp(buffer,"JUNK",4) == 0) {
		fread(&size,sizeof(int),1,moviefile);
		size = SWAP_BYTE_ORDER(size); // Convert to Little Endian Architecture
		fseek(moviefile,size,SEEK_CUR);
		fread(buffer,sizeof(char),4,moviefile); // chunck (LISTMOVIECHUNCK)
	}

	fread(&size,sizeof(int),1,moviefile);
	fread(buffer,sizeof(char),4,moviefile); // "movi"

	long* foffset = new long[frames];

	long totalsize = (width + (width&1))*height*3;

	for(uint i = 0; i < frames; ++i) {
		fread(buffer,sizeof(char),4,moviefile); // "00db"
		if (memcmp(buffer,"00db",4) != 0) {
			fclose(moviefile);
			exit(1);
		}

		fread(&size,sizeof(int),1,moviefile);
		size = SWAP_BYTE_ORDER(size); // Convert to Little Endian Architecture
		
		foffset[i] = ftell(moviefile);
		fseek(moviefile,totalsize,SEEK_CUR);
	}

	fileoffset = foffset;
	// keep file open for reading frame data
}

void avi::write_video(uint in_frames,uint in_width,uint in_height,double in_fps) {
	frames = in_frames;
	width = in_width;
	height = in_height;
	fps = in_fps;

	long size;
	long totalsize = (width + (width&1))*height*3;
	uint temp[1024];

	if((moviefile = fopen(name,"w+b")) == NULL) {
		exit(1);
	}

	fwrite("RIFF",sizeof(char),4,moviefile);
	size = (totalsize + 24)*frames + 228;
	size = SWAP_BYTE_ORDER(size); // Convert to Little Endian Architecture
	fwrite(&size,sizeof(int),1,moviefile);

	fwrite("AVI ",sizeof(char),4,moviefile);
	fwrite("LIST",sizeof(char),4,moviefile);

	size = 196;
	size = SWAP_BYTE_ORDER(size); // Convert to Little Endian Architecture
	fwrite(&size,sizeof(int),1,moviefile);
	fwrite("hdrl",sizeof(char),4,moviefile);

	fwrite("avih",sizeof(char),4,moviefile);
	size = 56;
	size = SWAP_BYTE_ORDER(size); // Convert to Little Endian Architecture
	fwrite(&size,sizeof(int),1,moviefile);

	// AVIHeader
	temp[0] = SWAP_BYTE_ORDER((uint)(1000000/fps)); // TimeBetweenFrames;
	temp[1] = 0; // MaximumDataRate;
	temp[2] = 0; // PaddingGranularity;
	temp[3] = SWAP_BYTE_ORDER(16); // Flags;
	temp[4] = SWAP_BYTE_ORDER((uint)(frames)); // TotalNumberOfFrames;
	temp[5] = 0; // NumberOfInitialFrames;
	temp[6] = SWAP_BYTE_ORDER(1); // NumberOfStreams;
	temp[7] = SWAP_BYTE_ORDER((uint)(totalsize)); // SuggestedBufferSize;
	temp[8] = SWAP_BYTE_ORDER((uint)(width)); // Width;
	temp[9] = SWAP_BYTE_ORDER((uint)(height)); // Height;
	temp[10] = SWAP_BYTE_ORDER(1); // TimeScale;
	temp[11] = SWAP_BYTE_ORDER((uint)(fps)); // DataRate;
	temp[12] = 0; // StartTime;
	temp[13] = SWAP_BYTE_ORDER((uint)(frames*fps)); // DataLength;
	fwrite(temp,56,1,moviefile);

	fwrite("LIST",sizeof(char),4,moviefile);
	size = 120;
	size = SWAP_BYTE_ORDER(size); // Convert to Little Endian Architecture
	fwrite(&size,sizeof(int),1,moviefile);
	fwrite("strl",sizeof(char),4,moviefile);

	fwrite("strh",sizeof(char),4,moviefile);
	size = 56;
	size = SWAP_BYTE_ORDER(size); // Convert to Little Endian Architecture
	fwrite(&size,sizeof(int),1,moviefile);

	//AVIStreamHeader
	temp[0] = SWAP_BYTE_ORDER(1935960438); // DataType[4] = "vids";
	temp[1] = SWAP_BYTE_ORDER(543319922); // DataHandler[4] = "rgb ";
	temp[2] = 0; // Flags;
	temp[3] = 2009006080; // Priority + Language;
	temp[4] = 0; // InitialFrames;
	temp[5] = SWAP_BYTE_ORDER((uint)(fps)); // TimeScale;
	temp[6] = SWAP_BYTE_ORDER((uint)(fps*fps)); // DataRate;
	temp[7] = 0; // StartTime;
	temp[8] = SWAP_BYTE_ORDER((uint)(frames)); // DataLength;
	temp[9] = SWAP_BYTE_ORDER((uint)(totalsize)); // SuggestedBufferSize;
	temp[10] = SWAP_BYTE_ORDER(-1); // Quality;
	temp[11] = 0; // SampleSize;
	temp[12] = 0; // Rectangle 1;
	temp[13] = SWAP_BYTE_ORDER((uint)(65536*height+width)); // Rectangle 2;
	fwrite(temp,56,1,moviefile);

	fwrite("strf",sizeof(char),4,moviefile);
	size = 44;
	size = SWAP_BYTE_ORDER(size); // Convert to Little Endian Architecture
	fwrite(&size,sizeof(int),1,moviefile);

	//AVIStreamFormatHeader
	temp[0] = SWAP_BYTE_ORDER(40); // FormatLength;
	temp[1] = SWAP_BYTE_ORDER((uint)(width)); // Width;
	temp[2] = SWAP_BYTE_ORDER((uint)(height)); // Height;
	temp[3] = SWAP_BYTE_ORDER(1572865); // NumberOfPlanes + BitCount;
	temp[4] = 0; // Type[4];
	temp[5] = SWAP_BYTE_ORDER((uint)(totalsize)); // PlaneSize;
	temp[6] = 0; // XperMeter;
	temp[7] = 0; // YperMeter;
	temp[8] = 0; // ClrUsed;
	temp[9] = 0; // ClrImportant;
	temp[10] = 0; // Colors[1];
	fwrite(temp,44,1,moviefile);

	fwrite("LIST",sizeof(char),4,moviefile);
	size = (totalsize + 8)*frames + 4;
	size = SWAP_BYTE_ORDER(size); // Convert to Little Endian Architecture
	fwrite(&size,sizeof(int),1,moviefile);

	fwrite("movi",sizeof(char),4,moviefile);

	long* foffset = new long[frames];
	foffset[0] = 236;
	size = width*height*3;
	long diffoffset = size + 8;
	size = SWAP_BYTE_ORDER(size); // Convert to Little Endian Architecture

	for(uint i = 1; i < frames; i++) {
		cout << "Pass " << i << endl;
		fwrite("00db",sizeof(char),4,moviefile);
		fwrite(&size,sizeof(int),1,moviefile);

		cout << " SEEK_CUR: " << SEEK_CUR << endl;
		cout << " totalsize: " << totalsize << endl;
		fseek(moviefile,totalsize,SEEK_CUR);

		foffset[i] = foffset[i - 1] + diffoffset;
		cout << " foffset: " << foffset[i] << endl;
	}

	// write last time
	fwrite("00db",sizeof(char),4,moviefile);
	fwrite(&size,sizeof(int),1,moviefile);

	fileoffset = foffset;

	fseek(moviefile,totalsize,SEEK_CUR);

	fwrite("idx1",sizeof(char),4,moviefile);
	size = 16*frames;
	size = SWAP_BYTE_ORDER(size); // Convert to Little Endian Architecture
	fwrite(&size,sizeof(int),1,moviefile);

	fwrite("00db",sizeof(char),4,moviefile);

	diffoffset -= 8;
	for(int i = 0; i < (int)(frames - 1); i++) {
		if (i == 0) {
			size = 16;
		} else {
			size = 0;
		}
		fwrite(&size,sizeof(int),1,moviefile);
		size = 4 + i*(diffoffset + 8);
		fwrite(&size,sizeof(int),1,moviefile);
		fwrite(&diffoffset,sizeof(int),1,moviefile);
		fwrite("00dc",sizeof(char),4,moviefile);
	}
	size = 0;
	fwrite(&size,sizeof(int),1,moviefile);
	size = 4 + (frames - 1)*(diffoffset + 8);
	fwrite(&size,sizeof(int),1,moviefile);
	fwrite(&diffoffset,sizeof(int),1,moviefile);

	// keep file open for writing frame data
}

frame avi::get_frame(uint index) {
	frame image(height,width);
	byte buffer[3];
	
	if (index < frames) {
		fseek(moviefile,(long)(fileoffset[index]),SEEK_SET);
		for(int i = height - 1; i >= 0; i--) {
			for (int j = 0; j < (int)(width); j++) {
				fread(buffer,sizeof(byte),3,moviefile);
				image(i,j).blue = (double)(SWAP_BYTE_ORDER(buffer[0]));
				image(i,j).green = (double)(SWAP_BYTE_ORDER(buffer[1]));
				image(i,j).red = (double)(SWAP_BYTE_ORDER(buffer[2]));
			}
			if (width&1) { // odd width
				fseek(moviefile,1,SEEK_CUR);
			}
		}
	}
	return image;
}

void avi::write_frame(uint index,frame& image) {
	byte buffer[3];
	fseek(moviefile,long(fileoffset[index]),SEEK_SET);

	if (index < frames) {
		for(int i = height - 1; i >= 0; i--) {
			for (int j = 0; j < (int)(width); j++) {
				buffer[0] = SWAP_BYTE_ORDER((byte)(ROUND(image(i,j).blue)));
				buffer[1] = SWAP_BYTE_ORDER((byte)(ROUND(image(i,j).green)));
				buffer[2] = SWAP_BYTE_ORDER((byte)(ROUND(image(i,j).red)));
				fwrite(&buffer,sizeof(byte),3,moviefile);
			}
			if (width&1) { // odd width
				fseek(moviefile,1,SEEK_CUR);
			}
		}
	}
}
