/*
Version: 0.97
Date: 15/03/05
*/

#ifndef DEFS_H
#define DEFS_H

#include <cstdlib>
#include <math.h>

// Conventions
typedef unsigned char byte;
typedef unsigned int uint;
typedef unsigned long long ulongint;

// Constants
#define M_PI 3.14159265358979323846 // Pi
#define M_E 2.7182818284590452354 // e
#define ZERO 1.0e-20 // floating point error

// Basic functions
template<class T> void SWAP(T& a,T& b) {
	T temp = a;
	a = b;
	b = temp;
}

template<class T> T MAX(T a,T b) {
	if (a > b) {
		return a;
	} else {
		return b;
	}
}

template<class T> T MIN(T a,T b) {
	if (a < b) {
		return a;
	} else {
		return b;
	}
}

#define ROUND(a) (floor(a + 0.5))
#define SIGN(a,b) ((b) >= 0.0 ? abs(a) : -abs(a))

#define RINT(a) ((int)(ROUND(a)))

// Colorspaces
class rgb {
public:
	double red,green,blue;
	rgb() {
		red = 0.0;
		green = 0.0;
		blue = 0.0;
	}
	rgb(double r,double g,double b) {
		red = r;
		green = g;
		blue = b;
	}
	~rgb() {
		red = 0.0;
		green = 0.0;
		blue = 0.0;
	}
	rgb & operator = (const rgb& z) {
		red = z.red;
		green = z.green;
		blue = z.blue;

		return *this;
	}
	rgb & operator = (const double& z) {
		red = z;
		green = z;
		blue = z;

		return *this;
	}
	rgb & operator += (const rgb& z) {
		red += z.red;
		green += z.green;
		blue += z.blue;

		return *this;
	}
	rgb & operator -= (const rgb& z) {
		red -= z.red;
		green -= z.green;
		blue -= z.blue;

		return *this;
	}
	rgb & operator += (const double& z) {
		red += z;
		green += z;
		blue += z;

		return *this;
	}
	rgb & operator -= (const double& z) {
		red -= z;
		green -= z;
		blue -= z;

		return *this;
	}
	rgb & operator *= (const double& z) {
		red *= z;
		green *= z;
		blue *= z;

		return *this;
	}
	rgb & operator /= (const double& z) {
		red /= z;
		green /= z;
		blue /= z;

		return *this;
	}
	rgb operator / (const double& z) {
		return rgb(red/z,green/z,blue/z);
	}
	rgb operator * (const double& z) {
		return rgb(z*red,z*green,z*blue);
	}
	friend rgb operator * (const double& z, const rgb& y) {
		return rgb(z*y.red,z*y.green,z*y.blue);
	}
	rgb operator + (const rgb& z) {
		return rgb(z.red + red,z.green + green,z.blue + blue);
	}
	rgb operator - (const rgb& z) {
		return rgb(red - z.red,green - z.green,blue - z.blue);
	}
};

class byte_rgb {
public:
	byte red,green,blue;
	byte_rgb() {
		red = 0;
		green = 0;
		blue = 0;
	}
	byte_rgb(byte r,byte g,byte b) {
		red = r;
		green = g;
		blue = b;
	}
	~byte_rgb() {
		red = 0;
		green = 0;
		blue = 0;
	}
	byte_rgb & operator = (const byte_rgb& z) {
		red = z.red;
		green = z.green;
		blue = z.blue;

		return *this;
	}
	byte_rgb & operator = (const rgb& z) {
		red = (byte)(RINT(MAX(MIN(z.red,255.0),0.0)));
		green = (byte)(RINT(MAX(MIN(z.green,255.0),0.0)));
		blue = (byte)(RINT(MAX(MIN(z.blue,255.0),0.0)));

		return *this;
	}
	byte_rgb & operator = (const int& z) {
		red = (byte)(z);
		green = (byte)(z);
		blue = (byte)(z);

		return *this;
	}
	byte_rgb & operator = (const double& z) {
		red = (byte)(RINT(MAX(MIN(z,255.0),0.0)));
		green = (byte)(RINT(MAX(MIN(z,255.0),0.0)));
		blue = (byte)(RINT(MAX(MIN(z,255.0),0.0)));

		return *this;
	}
	byte_rgb & operator += (const byte_rgb& z) {
		red += z.red;
		green += z.green;
		blue += z.blue;

		return *this;
	}
	byte_rgb & operator -= (const byte_rgb& z) {
		red -= z.red;
		green -= z.green;
		blue -= z.blue;

		return *this;
	}
	byte_rgb & operator += (const rgb& z) {
		red += RINT(z.red);
		green += RINT(z.green);
		blue += RINT(z.blue);

		return *this;
	}
	byte_rgb & operator -= (const rgb& z) {
		red -= RINT(z.red);
		green -= RINT(z.green);
		blue -= RINT(z.blue);

		return *this;
	}
	byte_rgb & operator += (const int& z) {
		red += z;
		green += z;
		blue += z;

		return *this;
	}
	byte_rgb & operator -= (const int& z) {
		red -= z;
		green -= z;
		blue -= z;

		return *this;
	}
	byte_rgb & operator *= (const int& z) {
		red *= z;
		green *= z;
		blue *= z;

		return *this;
	}
	byte_rgb & operator /= (const int& z) {
		red /= z;
		green /= z;
		blue /= z;

		return *this;
	}
	byte_rgb & operator += (const double& z) {
		red += RINT(z);
		green += RINT(z);
		blue += RINT(z);

		return *this;
	}
	byte_rgb & operator -= (const double& z) {
		red -= RINT(z);
		green -= RINT(z);
		blue -= RINT(z);

		return *this;
	}
	byte_rgb & operator *= (const double& z) {
		red = RINT(red*z);
		green = RINT(green*z);
		blue = RINT(blue*z);

		return *this;
	}
	byte_rgb & operator /= (const double& z) {
		red = RINT(red/z);
		green = RINT(green/z);
		blue = RINT(blue/z);

		return *this;
	}
	byte_rgb operator / (const double& z) {
		return byte_rgb(RINT(red/z),RINT(green/z),RINT(blue/z));
	}
	byte_rgb operator * (const double& z) {
		return byte_rgb(RINT(z*red),RINT(z*green),RINT(z*blue));
	}
	friend byte_rgb operator * (const double& z, const byte_rgb& y) {
		return byte_rgb(RINT(z*y.red),RINT(z*y.green),RINT(z*y.blue));
	}
	byte_rgb operator + (const byte_rgb& z) {
		return byte_rgb(z.red + red,z.green + green,z.blue + blue);
	}
	byte_rgb operator - (const byte_rgb& z) {
		return byte_rgb(red - z.red,green - z.green,blue - z.blue);
	}
};

const rgb black(0.0,0.0,0.0);
const rgb white(255.0,255.0,255.0);
const rgb red(255.0,0.0,0.0);
const rgb green(0.0,255.0,0.0);
const rgb blue(0.0,0.0,255.0);
const rgb yellow(255.0,255.0,0.0);

template<class T>
class vector {
public:
    int length;

	vector() {
		length = 0;
		objectlist = NULL;
	}
	vector(int size) {
		length = size;
		objectlist = new T [length];
	}
	~vector() {delete [] objectlist;}
	vector(const vector& v) {
		length = v.length;
		objectlist = new T [length];
		for (int i = 0; i < length; i++) {
			objectlist[i] = v.objectlist[i];
		}
	}
	vector & operator = (const vector& v) {
		if (this == &v) return *this;
		if (length != v.length) {
			delete [] objectlist;
			length = v.length;
			objectlist = new T [length];
		}
		for (int i = 0; i < length; i++) {
			objectlist[i] = v.objectlist[i];
		}
		return *this;
	}
	T & operator () (int index) {
		return objectlist[index];
	}

private:
	T* objectlist;
};

template<class T>
class matrix {
public:
    int height,width;
	matrix() {
		height = 0;
		width = 0;
		datamatrix = NULL;
		objectmatrix = NULL;
	}
	matrix(int dim1,int dim2) {
		height = dim1;
		width = dim2;
		datamatrix = new T [height*width];
		objectmatrix = new T* [height];

		for (int i = 0; i < height; i++) {
			objectmatrix[i] = datamatrix + i*width;
		}
	}
	~matrix() {
		delete [] objectmatrix;
		delete [] datamatrix;
	}
	matrix(const matrix& m) {
		height = m.height;
		width = m.width;
		datamatrix = new T [height*width];
		objectmatrix = new T* [height];

		for (int i = 0; i < height; i++) {
			objectmatrix[i] = datamatrix + i*width;
		}
		for (int i = 0; i < height; i++) {
			for (int j = 0; j < width; j++) {
				objectmatrix[i][j] = m.objectmatrix[i][j];
			}
		}
	}
	matrix & operator = (const matrix& m) {
		if (this == &m) return *this;
		if (height != m.height || width != m.width) {
			delete [] objectmatrix;
			delete [] datamatrix;
			height = m.height;
			width = m.width;
			datamatrix = new T [height*width];
			objectmatrix = new T* [height];

			for (int i = 0; i < height; i++) {
				objectmatrix[i] = datamatrix + i*width;
			}
		}
		for (int i = 0; i < height; i++) {
			for (int j = 0; j < width; j++) {
				objectmatrix[i][j] = m.objectmatrix[i][j];
			}
		}
		return *this;
	}
	T & operator () (int index1,int index2) {
		/*
		// mirror feature
		int ind1 = abs(index1);
		int ind2 = abs(index2);

		if (index1 >= height) {
			ind1 = 2*(height - 1) - index1;
		}
		if (index2 >= width) {
			ind2 = 2*(width - 1) - index2;
		}
		*/

		// boundary feature
		int ind1 = index1;
		int ind2 = index2;

		if (index1 < 0) {
			ind1 = 0;
		}
		if (index2 < 0) {
			ind2 = 0;
		}

		if (index1 >= height) {
			ind1 = height - 1;
		}
		if (index2 >= width) {
			ind2 = width - 1;
		}

		return objectmatrix[ind1][ind2];
	}

	matrix & operator += (const matrix& m) {
		for (int i = 0; i < height; i++) {
			for (int j = 0; j < width; j++) {
				objectmatrix[i][j] += m.objectmatrix[i][j];
			}
		}
		return *this;
	}
	matrix & operator -= (const matrix& m) {
		for (int i = 0; i < height; i++) {
			for (int j = 0; j < width; j++) {
				objectmatrix[i][j] -= m.objectmatrix[i][j];
			}
		}
		return *this;
	}
	matrix & operator += (const double& z) {
		for (int i = 0; i < height; i++) {
			for (int j = 0; j < width; j++) {
				objectmatrix[i][j] += z;
			}
		}
		return *this;
	}
	matrix & operator -= (const double& z) {
		for (int i = 0; i < height; i++) {
			for (int j = 0; j < width; j++) {
				objectmatrix[i][j] -= z;
			}
		}
		return *this;
	}
	matrix & operator *= (const double& z) {
		for (int i = 0; i < height; i++) {
			for (int j = 0; j < width; j++) {
				objectmatrix[i][j] *= z;
			}
		}
		return *this;
	}
	matrix & operator /= (const double& z) {
		for (int i = 0; i < height; i++) {
			for (int j = 0; j < width; j++) {
				objectmatrix[i][j] /= z;
			}
		}
		return *this;
	}
	matrix operator * (const double& z) {
		matrix m2 = *this;
		for (int i = 0; i < height; i++) {
			for (int j = 0; j < width; j++) {
				m2.objectmatrix[i][j] *= z;
			}
		}
		return m2;
	}
	matrix operator / (const double& z) {
		matrix m2 = *this;
		for (int i = 0; i < height; i++) {
			for (int j = 0; j < width; j++) {
				m2.objectmatrix[i][j] /= z;
			}
		}
		return m2;
	}
	matrix operator + (const matrix& m) {
		matrix m2 = *this;
		for (int i = 0; i < height; i++) {
			for (int j = 0; j < width; j++) {
				m2.objectmatrix[i][j] = objectmatrix[i][j] + m.objectmatrix[i][j];
			}
		}
		return m2;
	}
	matrix operator - (const matrix& m) {
		matrix m2 = *this;
		for (int i = 0; i < height; i++) {
			for (int j = 0; j < width; j++) {
				m2.objectmatrix[i][j] = objectmatrix[i][j] - m.objectmatrix[i][j];
			}
		}
		return m2;
	}
	friend matrix operator * (const double& z, const matrix& m) {
		matrix m2 = m;
		for (int i = 0; i < m2.height; i++) {
			for (int j = 0; j < m2.width; j++) {
				m2.objectmatrix[i][j] *= z;
			}
		}
		return m2;
	}
private:
	T** objectmatrix;
	T* datamatrix;
};

template<class T>
class cube {
public:
    int height,width,length;
	cube() {
		height = 0;
		width = 0;
		length = 0;
		datamatrix = NULL;
	}
	cube(int dim1,int dim2,int dim3) {
		height = dim1;
		width = dim2;
		length = dim3;
		datamatrix = new T [height*width*length];
	}
	~cube() {
		delete [] datamatrix;
	}
	cube & operator = (const cube& m) {
		if (this == &m) return *this;
		if (height != m.height || width != m.width || length != m.length) {
			delete [] datamatrix;
			height = m.height;
			width = m.width;
			length = m.length;
			datamatrix = new T [height*width*length];
		}
		int i2 = 0;
		int j2 = 0;
		for (int i = 0; i < height; i++) {
			i2 = i*width*length;
			for (int j = 0; j < width; j++) {
				j2 = j*length;
				for (int k = 0; k < length; k++) {
					datamatrix[i2 + j2 + k] = m.datamatrix[i2 + j2 + k];
				}
			}
		}
		return *this;
	}
	T & operator () (int index1,int index2,int index3) {
		return datamatrix[index1*width*length + index2*length + index3];
	}

private:
	T* datamatrix;
};

typedef struct {
	double x;
	double y;
} coordinate;

#define norm_coordinate(a) (sqrt(a.x*a.x + a.y*a.y))

typedef matrix<rgb> frame;
typedef matrix<byte_rgb> byte_frame;

typedef matrix<byte> bytematrix;
typedef matrix<int> intmatrix;
typedef matrix<double> doublematrix;
typedef matrix<coordinate> coordinatematrix;

typedef vector<byte> bytevector;
typedef vector<int> intvector;
typedef vector<double> doublevector;
typedef vector<coordinate> coordinatevector;

#endif // DEFS_H
