#include "filter.h"

frame filter_robust_regularization(frame& image,byte type1,byte type2,double sigma,double lambda,int maxiter,double step) {
	frame result = image;
	frame update_data(image.height,image.width);
	frame update_regularization(image.height,image.width);
	doublevector kernel_g = gaussian_derivative(15,0,sigma);
	rgb pixel;
	int intpar1;
	double doublepar1;
	vector<rgb> avector(4); // case 9
	vector<rgb> rvector(4); // case 9
	doublevector dvector; // case 10

	for (int iter = 0; iter < maxiter; iter++) {
		// data fidelity term
		update_data = convolution_separable(result,kernel_g,kernel_g);
		switch (type1) {
			case 0:
				for (int j = 0; j < image.height; j++) {
					for (int i = 0; i < image.width; i++) {
						pixel = image(j,i) - update_data(j,i);
						update_data(j,i) = pixel;
					}
				}
				break;
			case 1:
				for (int j = 0; j < image.height; j++) {
					for (int i = 0; i < image.width; i++) {
						pixel = image(j,i) - update_data(j,i);
						if (fabs(pixel.red) < ZERO) {
							update_data(j,i).red = 0.0;
						} else {
							if (pixel.red > 0.0) {
								update_data(j,i).red = 1.0;
							} else {
								update_data(j,i).red = -1.0;
							}
						}
						if (fabs(pixel.green) < ZERO) {
							update_data(j,i).green = 0.0;
						} else {
							if (pixel.green > 0.0) {
								update_data(j,i).green = 1.0;
							} else {
								update_data(j,i).green = -1.0;
							}
						}
						if (fabs(pixel.blue) < ZERO) {
							update_data(j,i).blue = 0.0;
						} else {
							if (pixel.blue > 0.0) {
								update_data(j,i).blue = 1.0;
							} else {
								update_data(j,i).blue = -1.0;
							}
						}
					}
				}
				break;
		}
		update_data = convolution_separable(update_data,kernel_g,kernel_g);

		// regularization term
		switch (type2) {
			case 0: // Tikhonov
				for (int j = 0; j < image.height; j++) {
					for (int i = 0; i < image.width; i++) {
						update_regularization(j,i) = result(j - 1,i) + result(j,i - 1) + result(j + 1,i) + result(j,i + 1) - 4*result(j,i);
					}
				}
				break;
			case 1: // Total Variation
				for (int j = 0; j < image.height; j++) {
					for (int i = 0; i < image.width; i++) {
						update_regularization(j,i) = black;
						pixel = result(j - 1,i) - result(j,i);
						if (fabs(pixel.red) < ZERO) {
							update_regularization(j,i).red += 0.0;
						} else {
							if (pixel.red > 0.0) {
								update_regularization(j,i).red += 1.0;
							} else {
								update_regularization(j,i).red += -1.0;
							}
						}
						if (fabs(pixel.green) < ZERO) {
							update_regularization(j,i).green += 0.0;
						} else {
							if (pixel.green > 0.0) {
								update_regularization(j,i).green += 1.0;
							} else {
								update_regularization(j,i).green += -1.0;
							}
						}
						if (fabs(pixel.blue) < ZERO) {
							update_regularization(j,i).blue += 0.0;
						} else {
							if (pixel.blue > 0.0) {
								update_regularization(j,i).blue += 1.0;
							} else {
								update_regularization(j,i).blue += -1.0;
							}
						}

						pixel = result(j,i - 1) - result(j,i);
						if (fabs(pixel.red) < ZERO) {
							update_regularization(j,i).red += 0.0;
						} else {
							if (pixel.red > 0.0) {
								update_regularization(j,i).red += 1.0;
							} else {
								update_regularization(j,i).red += -1.0;
							}
						}
						if (fabs(pixel.green) < ZERO) {
							update_regularization(j,i).green += 0.0;
						} else {
							if (pixel.green > 0.0) {
								update_regularization(j,i).green += 1.0;
							} else {
								update_regularization(j,i).green += -1.0;
							}
						}
						if (fabs(pixel.blue) < ZERO) {
							update_regularization(j,i).blue += 0.0;
						} else {
							if (pixel.blue > 0.0) {
								update_regularization(j,i).blue += 1.0;
							} else {
								update_regularization(j,i).blue += -1.0;
							}
						}

						pixel = result(j + 1,i) - result(j,i);
						if (fabs(pixel.red) < ZERO) {
							update_regularization(j,i).red += 0.0;
						} else {
							if (pixel.red > 0.0) {
								update_regularization(j,i).red += 1.0;
							} else {
								update_regularization(j,i).red += -1.0;
							}
						}
						if (fabs(pixel.green) < ZERO) {
							update_regularization(j,i).green += 0.0;
						} else {
							if (pixel.green > 0.0) {
								update_regularization(j,i).green += 1.0;
							} else {
								update_regularization(j,i).green += -1.0;
							}
						}
						if (fabs(pixel.blue) < ZERO) {
							update_regularization(j,i).blue += 0.0;
						} else {
							if (pixel.blue > 0.0) {
								update_regularization(j,i).blue += 1.0;
							} else {
								update_regularization(j,i).blue += -1.0;
							}
						}

						pixel = result(j,i + 1) - result(j,i);
						if (fabs(pixel.red) < ZERO) {
							update_regularization(j,i).red += 0.0;
						} else {
							if (pixel.red > 0.0) {
								update_regularization(j,i).red += 1.0;
							} else {
								update_regularization(j,i).red += -1.0;
							}
						}
						if (fabs(pixel.green) < ZERO) {
							update_regularization(j,i).green += 0.0;
						} else {
							if (pixel.green > 0.0) {
								update_regularization(j,i).green += 1.0;
							} else {
								update_regularization(j,i).green += -1.0;
							}
						}
						if (fabs(pixel.blue) < ZERO) {
							update_regularization(j,i).blue += 0.0;
						} else {
							if (pixel.blue > 0.0) {
								update_regularization(j,i).blue += 1.0;
							} else {
								update_regularization(j,i).blue += -1.0;
							}
						}
					}
				}
				break;
			case 2: // Bilateral Total Variation
				doublepar1 = 0.4316834166; // [0,1]  with step = 0.25 : 0.6180339887 (w = 1) / 0.4316834166 (w = 2) / 0.3973766961 (w = 3) / 0.3873556790 (w = 4)
				intpar1 = 2;
				for (int j = 0; j < image.height; j++) {
					for (int i = 0; i < image.width; i++) {
						update_regularization(j,i) = black;
						for (int l = -intpar1; l <= intpar1; l++) {
							for (int k = -intpar1; k <= intpar1; k++) {
								pixel = result(j + l,i + k) - result(j,i);
								if (fabs(pixel.red) < ZERO) {
									update_regularization(j,i).red += 0.0;
								} else {
									if (pixel.red > 0.0) {
										update_regularization(j,i).red += pow(doublepar1,abs(k) + abs(l));
									} else {
										update_regularization(j,i).red += -pow(doublepar1,abs(k) + abs(l));
									}
								}
								if (fabs(pixel.green) < ZERO) {
									update_regularization(j,i).green += 0.0;
								} else {
									if (pixel.green > 0.0) {
										update_regularization(j,i).green += pow(doublepar1,abs(k) + abs(l));
									} else {
										update_regularization(j,i).green += -pow(doublepar1,abs(k) + abs(l));
									}
								}
								if (fabs(pixel.blue) < ZERO) {
									update_regularization(j,i).blue += 0.0;
								} else {
									if (pixel.blue > 0.0) {
										update_regularization(j,i).blue += pow(doublepar1,abs(k) + abs(l));
									} else {
										update_regularization(j,i).blue += -pow(doublepar1,abs(k) + abs(l));
									}
								}
							}
						}
					}
				}
				break;
			case 3: // Lorentzian (Perona-Malik)
				doublepar1 = 1.0; // 5.0*5.0
				for (int j = 0; j < image.height; j++) {
					for (int i = 0; i < image.width; i++) {
						update_regularization(j,i) = black;
						pixel = result(j - 1,i) - result(j,i);
						update_regularization(j,i).red += (1.0/(1.0 + (pixel.red*pixel.red)/doublepar1))*pixel.red;
						update_regularization(j,i).green += (1.0/(1.0 + (pixel.green*pixel.green)/doublepar1))*pixel.green;
						update_regularization(j,i).blue += (1.0/(1.0 + (pixel.blue*pixel.blue)/doublepar1))*pixel.blue;

						pixel = result(j,i - 1) - result(j,i);
						update_regularization(j,i).red += (1.0/(1.0 + (pixel.red*pixel.red)/doublepar1))*pixel.red;
						update_regularization(j,i).green += (1.0/(1.0 + (pixel.green*pixel.green)/doublepar1))*pixel.green;
						update_regularization(j,i).blue += (1.0/(1.0 + (pixel.blue*pixel.blue)/doublepar1))*pixel.blue;

						pixel = result(j + 1,i) - result(j,i);
						update_regularization(j,i).red += (1.0/(1.0 + (pixel.red*pixel.red)/doublepar1))*pixel.red;
						update_regularization(j,i).green += (1.0/(1.0 + (pixel.green*pixel.green)/doublepar1))*pixel.green;
						update_regularization(j,i).blue += (1.0/(1.0 + (pixel.blue*pixel.blue)/doublepar1))*pixel.blue;

						pixel = result(j,i + 1) - result(j,i);
						update_regularization(j,i).red += (1.0/(1.0 + (pixel.red*pixel.red)/doublepar1))*pixel.red;
						update_regularization(j,i).green += (1.0/(1.0 + (pixel.green*pixel.green)/doublepar1))*pixel.green;
						update_regularization(j,i).blue += (1.0/(1.0 + (pixel.blue*pixel.blue)/doublepar1))*pixel.blue;
					}
				}
				break;
			case 4: // Le Clerc (Perona-Malik)
				doublepar1 = 100.0;
				for (int j = 0; j < image.height; j++) {
					for (int i = 0; i < image.width; i++) {
						update_regularization(j,i) = black;
						pixel = result(j - 1,i) - result(j,i);
						update_regularization(j,i).red += exp(-(pixel.red*pixel.red)/doublepar1)*pixel.red;
						update_regularization(j,i).green += exp(-(pixel.green*pixel.green)/doublepar1)*pixel.green;
						update_regularization(j,i).blue += exp(-(pixel.blue*pixel.blue)/doublepar1)*pixel.blue;

						pixel = result(j,i - 1) - result(j,i);
						update_regularization(j,i).red += exp(-(pixel.red*pixel.red)/doublepar1)*pixel.red;
						update_regularization(j,i).green += exp(-(pixel.green*pixel.green)/doublepar1)*pixel.green;
						update_regularization(j,i).blue += exp(-(pixel.blue*pixel.blue)/doublepar1)*pixel.blue;

						pixel = result(j + 1,i) - result(j,i);
						update_regularization(j,i).red += exp(-(pixel.red*pixel.red)/doublepar1)*pixel.red;
						update_regularization(j,i).green += exp(-(pixel.green*pixel.green)/doublepar1)*pixel.green;
						update_regularization(j,i).blue += exp(-(pixel.blue*pixel.blue)/doublepar1)*pixel.blue;

						pixel = result(j,i + 1) - result(j,i);
						update_regularization(j,i).red += exp(-(pixel.red*pixel.red)/doublepar1)*pixel.red;
						update_regularization(j,i).green += exp(-(pixel.green*pixel.green)/doublepar1)*pixel.green;
						update_regularization(j,i).blue += exp(-(pixel.blue*pixel.blue)/doublepar1)*pixel.blue;
					}
				}
				break;
			case 5: // Tukey Bi-weight
				doublepar1 = sqrt(5.0)*10.0; // 20.0
				for (int j = 0; j < image.height; j++) {
					for (int i = 0; i < image.width; i++) {
						update_regularization(j,i) = black;
						pixel = result(j - 1,i) - result(j,i);
						if (fabs(pixel.red) < doublepar1) {
							update_regularization(j,i).red += pow((1.0 - (pixel.red*pixel.red)/(doublepar1*doublepar1)),2)*pixel.red;
						}
						if (fabs(pixel.green) < doublepar1) {
							update_regularization(j,i).green += pow((1.0 - (pixel.green*pixel.green)/(doublepar1*doublepar1)),2)*pixel.green;
						}
						if (fabs(pixel.blue) < doublepar1) {
							update_regularization(j,i).blue += pow((1.0 - (pixel.blue*pixel.blue)/(doublepar1*doublepar1)),2)*pixel.blue;
						}

						pixel = result(j,i - 1) - result(j,i);
						if (fabs(pixel.red) < doublepar1) {
							update_regularization(j,i).red += pow((1.0 - (pixel.red*pixel.red)/(doublepar1*doublepar1)),2)*pixel.red;
						}
						if (fabs(pixel.green) < doublepar1) {
							update_regularization(j,i).green += pow((1.0 - (pixel.green*pixel.green)/(doublepar1*doublepar1)),2)*pixel.green;
						}
						if (fabs(pixel.blue) < doublepar1) {
							update_regularization(j,i).blue += pow((1.0 - (pixel.blue*pixel.blue)/(doublepar1*doublepar1)),2)*pixel.blue;
						}

						pixel = result(j + 1,i) - result(j,i);
						if (fabs(pixel.red) < doublepar1) {
							update_regularization(j,i).red += pow((1.0 - (pixel.red*pixel.red)/(doublepar1*doublepar1)),2)*pixel.red;
						}
						if (fabs(pixel.green) < doublepar1) {
							update_regularization(j,i).green += pow((1.0 - (pixel.green*pixel.green)/(doublepar1*doublepar1)),2)*pixel.green;
						}
						if (fabs(pixel.blue) < doublepar1) {
							update_regularization(j,i).blue += pow((1.0 - (pixel.blue*pixel.blue)/(doublepar1*doublepar1)),2)*pixel.blue;
						}

						pixel = result(j,i + 1) - result(j,i);
						if (fabs(pixel.red) < doublepar1) {
							update_regularization(j,i).red += pow((1.0 - (pixel.red*pixel.red)/(doublepar1*doublepar1)),2)*pixel.red;
						}
						if (fabs(pixel.green) < doublepar1) {
							update_regularization(j,i).green += pow((1.0 - (pixel.green*pixel.green)/(doublepar1*doublepar1)),2)*pixel.green;
						}
						if (fabs(pixel.blue) < doublepar1) {
							update_regularization(j,i).blue += pow((1.0 - (pixel.blue*pixel.blue)/(doublepar1*doublepar1)),2)*pixel.blue;
						}
					}
				}
				break;			
		}

		// diffusion
		for (int j = 0; j < image.height; j++) {
			for (int i = 0; i < image.width; i++) {
				result(j,i) += step*(update_regularization(j,i) + lambda*update_data(j,i));
				result(j,i).red = MAX(MIN(result(j,i).red,255.0),0.0);
				result(j,i).green = MAX(MIN(result(j,i).green,255.0),0.0);
				result(j,i).blue = MAX(MIN(result(j,i).blue,255.0),0.0);
			}
		}
	}

	return result;
}

