#include "image.h"

using namespace std;

//=============================================================================
// convolution
//=============================================================================
//    Berekent de convolutie van een beeld met een gegeven kernel
//    Het resultaat heeft dezelfde afmetingen als het inputbeeld
//=============================================================================
frame convolution(frame& image,doublematrix& kernel) {
	frame result(image.height,image.width);
	rgb temp;
	int height = kernel.height >> 1;
	int width = kernel.width >> 1;

	for (int i = 0; i < image.height; i++) {
		for (int j = 0; j < image.width; j++) {
			temp = black;
			for (int k = -height; k <= height; k++) {
				for (int l = -width; l <= width; l++) {
					temp += image(i + k,j + l)*kernel(height - k,width - l);
				}
			}
			result(i,j) = temp;
		}
	}

	return result;
}

//=============================================================================
// convolution_seperable
//=============================================================================
//    Berekent de scheidbare convolutie van een beeld met twee kernels
//    (de eerste horizontaal, de tweede verticaal)
//    Het resultaat heeft dezelfde afmetingen als het inputbeeld
//=============================================================================
frame convolution_separable(frame& image,doublevector& kernel1,doublevector& kernel2) {
	frame result(image.height,image.width);
	frame tempimage(image.height,image.width);
	rgb temp;
	int length = kernel1.length >> 1;

	for (int i = 0; i < image.height; i++) {
		for (int j = 0; j < image.width; j++) {
			temp = black;
			for (int k = -length; k <= length; k++) {
				temp += image(i,j + k)*kernel1(length - k);
			}
			tempimage(i,j) = temp;
		}
	}

	length = kernel2.length >> 1;

	for (int i = 0; i < image.width; i++) {
		for (int j = 0; j < image.height; j++) {
			temp = black;
			for (int k = -length; k <= length; k++) {
				temp += tempimage(j + k,i)*kernel2(length - k);
			}
			result(j,i) = temp;
		}
	}

	return result;
}

//=============================================================================
// convert_rgb2grey
//=============================================================================
//     Omzetten van een beeld naar grijswaarden
//=============================================================================
doublematrix convert_rgb2grey(frame& image) {
	doublematrix result(image.height,image.width);

	for (int i = 0; i < image.height; i++) {
		for (int j = 0; j < image.width; j++) {
			result(i,j) = rgb2grey(image(i,j));
		}
	}

	return result;
}
