#include <iostream>
#include "interpolation.h"

using namespace std;

//=============================================================================
// interpolation_bspline_cubic 
//=============================================================================
//    Cubic B-spline interpolatie van een beeld
// Argumenten:
//    - height: hoogte van het nieuwe beeld
//	  - width:  breedte van het nieuwe beeld
//=============================================================================
// Opmerkingen: voor het project moeten er aan deze functie geen aanpassingen
// gedaan worden.
//=============================================================================
frame interpolation_bspline_cubic(frame& image,int height,int width) {
	frame result(height,width);

	frame coefficients = image;

	double step_y = (double)(image.height)/(double)(height);
	double step_x = (double)(image.width)/(double)(width);

	doublevector position_x(width);
	doublevector position_y(height);

	double delta;
	int pos1,pos2,pos3,pos4;
	rgb p1,p2,p3,p4,a,b,c,d;

	position_y(0) = (step_y - 1.0)/2.0;
	for (int i = 1; i < height; i++) {
		position_y(i) = position_y(i - 1) + step_y;
	}

	position_x(0) = (step_x - 1.0)/2.0;
	for (int i = 1; i < width; i++) {
		position_x(i) = position_x(i - 1) + step_x;
	}

	// Calculate coefficients
	doublematrix tri_height(image.height,image.height);
	doublematrix tri_width(image.width,image.width);

	for (int j = 0; j < tri_height.width; j++) {
		for (int i = 0; i < tri_height.height; i++) {
			tri_height(j,i) = 0.0;
		}
		tri_height(j,j) = 4.0/6.0;
	}
	for (int j = 0; j < (tri_height.width - 1); j++) {
		tri_height(j,j + 1) = 1.0/6.0;
		tri_height(j + 1,j) = 1.0/6.0;
	}
	tri_height(0,0) += 1.0/6.0;
	tri_height(tri_height.height - 1,tri_height.width - 1) += 1.0/6.0;

	for (int j = 0; j < tri_width.width; j++) {
		for (int i = 0; i < tri_width.height; i++) {
			tri_width(j,i) = 0.0;
		}
		tri_width(j,j) = 4.0/6.0;
	}
	for (int j = 0; j < (tri_width.width - 1); j++) {
		tri_width(j,j + 1) = 1.0/6.0;
		tri_width(j + 1,j) = 1.0/6.0;
	}
	tri_width(0,0) += 1.0/6.0;
	tri_width(tri_width.height - 1,tri_width.width - 1) += 1.0/6.0;

	doublematrix inv_tri_height = inverse(tri_height);
	doublematrix inv_tri_width = inverse(tri_width);

	doublematrix row(1,image.width);
	doublematrix col(image.height,1);

	for (int j = 0; j < image.height; j++) {
		for (int i = 0; i < image.width; i++) {
			row(0,i) = coefficients(j,i).red;
		}
		row = product(row,inv_tri_width);
		for (int i = 0; i < image.width; i++) {
			coefficients(j,i).red = row(0,i);
		}

		for (int i = 0; i < image.width; i++) {
			row(0,i) = coefficients(j,i).green;
		}
		row = product(row,inv_tri_width);
		for (int i = 0; i < image.width; i++) {
			coefficients(j,i).green = row(0,i);
		}

		for (int i = 0; i < image.width; i++) {
			row(0,i) = coefficients(j,i).blue;
		}
		row = product(row,inv_tri_width);
		for (int i = 0; i < image.width; i++) {
			coefficients(j,i).blue = row(0,i);
		}
	}

	for (int i = 0; i < image.width; i++) {
		for (int j = 0; j < image.height; j++) {
			 col(j,0) = coefficients(j,i).red;
		}
		col = product(inv_tri_height,col);
		for (int j = 0; j < image.height; j++) {
			coefficients(j,i).red = col(j,0);
		}

		for (int j = 0; j < image.height; j++) {
			 col(j,0) = coefficients(j,i).green;
		}
		col = product(inv_tri_height,col);
		for (int j = 0; j < image.height; j++) {
			coefficients(j,i).green = col(j,0);
		}

		for (int j = 0; j < image.height; j++) {
			 col(j,0) = coefficients(j,i).blue;
		}
		col = product(inv_tri_height,col);
		for (int j = 0; j < image.height; j++) {
			coefficients(j,i).blue = col(j,0);
		}
	}

	frame image_temp(image.height,width);

	// x-transform
	for (int i = 0; i < width; i++) {
		pos1 = (int)(floor(position_x(i))) - 1;
		pos2 = (int)(floor(position_x(i)));
		pos3 = (int)(floor(position_x(i))) + 1;
		pos4 = (int)(floor(position_x(i))) + 2;
		delta = position_x(i) - floor(position_x(i));
		for (int j = 0; j < image.height; j++) {
			p1 = coefficients(j,pos1);
			p2 = coefficients(j,pos2);
			p3 = coefficients(j,pos3);
			p4 = coefficients(j,pos4);

			a = -(1.0/6.0)*p1 + 0.5*p2 - 0.5*p3 + (1.0/6.0)*p4;
			b = 0.5*p1 - p2 + 0.5*p3;
			c = -0.5*p1 + 0.5*p3;
			d = (1.0/6.0)*p1 + (2.0/3.0)*p2 + (1.0/6.0)*p3;

			image_temp(j,i) = a*pow(delta,3) + b*pow(delta,2) + c*delta + d;
		}
	}

	// y-transform
	for (int j = 0; j < height; j++) {
		pos1 = (int)(floor(position_y(j))) - 1;
		pos2 = (int)(floor(position_y(j)));
		pos3 = (int)(floor(position_y(j))) + 1;
		pos4 = (int)(floor(position_y(j))) + 2;
		delta = position_y(j) - floor(position_y(j));
		for (int i = 0; i < width; i++) {
			p1 = image_temp(pos1,i);
			p2 = image_temp(pos2,i);
			p3 = image_temp(pos3,i);
			p4 = image_temp(pos4,i);

			a = -(1.0/6.0)*p1 + 0.5*p2 - 0.5*p3 + (1.0/6.0)*p4;
			b = 0.5*p1 - p2 + 0.5*p3;
			c = -0.5*p1 + 0.5*p3;
			d = (1.0/6.0)*p1 + (2.0/3.0)*p2 + (1.0/6.0)*p3;

			result(j,i) = a*pow(delta,3) + b*pow(delta,2) + c*delta + d;
		}
	}

	return result;
}
