#include "ppm.h"
#include "defs.h"
#include "image.h"
#include "matrix.h"
#include "interpolation.h"
#include "superresolution.h"
#include "color.h"
#include "filter.h"

#include <time.h>
#include <iostream>
#include <fstream>

#include <stdlib.h>

using namespace std;

//=============================================================================
// main
//=============================================================================
//   Het hoofdprogramma
//=============================================================================
int main(int argc,char *argv[]) {
	clock_t start,stop;

	start = clock();
	char filename[255];

	cout << "Basis Super Resolution tool 0.2 beta (2008/09/25)" << endl;
	cout << "(voor practicum Beeldverwerking)" << endl;

	if (argc != 8) {
		cout << "Contact: Hiep Luong <hiep.luong@telin.UGent.be>" << endl;
		cout << "         Bart Goossens <bart.goossens@telin.UGent.be>" << endl;
		cout << "Gebruik: " << argv[0] << " <INPUTSEQUENTIE> <VERGROTINGSFACTOR> <TYPE DEBLURRING/DENOISING> <SIGMA> <LAMBDA> <STAPGROOTTE> <ITERATIES>" << endl;
		cout << " - INPUTSEQUENTIE: <string> tekstbestand met 'beginnummer eindnummer naam%d.ppm'" << endl;
		cout << " - VERGROTINGSFACTOR: <integer> magnificatie van MxN naar sMxSN pixels" << endl;
		cout << " - TYPE D/D: <integer> type van databehoudsterm + regularisatieterm" << endl;
		cout << "   > 0/else: L2 + Tikhonov" << endl;
		cout << "   > 1: L1 + Total Variation" << endl;
		cout << "   > 2: L1 + Bilateral Total Variation" << endl;
		cout << "   > 3: L1 + Lorentzian" << endl;
		cout << " - SIGMA: <double> standaardafwijking van de Gaussiaanse blurkernel" << endl;
		cout << " - LAMBDA: <double> regularisatie tussen databehoudsterm en regularisatieterm" << endl;
		cout << " - STAPGROOTTE: <double> stapgrootte in de steepest descent richting" << endl;
		cout << " - ITERATIES: <integer> aantal steepest descent stappen" << endl;
		return 1;
	}
	
	// Volledige videosequentie inlezen
	ppmsequence input(argv[1]);
	input.read_video();

	// Inlezen van de command line argumenten
	int factor = atoi(argv[2]);
	byte type = atoi(argv[3]);
	double sigma = atof(argv[4]);
	double lambda = atof(argv[5]);
	double step = atof(argv[6]);
	int iterations = atoi(argv[7]);

	// Het echte werk gebeurt hier:
	frame im = superresolution(input,0,factor,type,sigma,lambda,step,iterations);

	// Wegschrijven van het eindresultaat naar een .ppm bestand
	strcpy(filename,argv[1]);
	strcat(filename,"_superresolution.ppm");
	write_ppm(filename,im);

	// Interpoleer het eerste lage resolutiebeeld via cubic B-splines, als referentiemethode
	// om mee te vergelijken (dit is uiteraard geen superresolutie!)
	frame inputframe = input.get_frame(0);
	im = interpolation_bspline_cubic(inputframe,input.height*factor,input.width*factor);
	strcpy(filename,argv[1]);
	strcat(filename,"_cubicbspline.ppm");
	write_ppm(filename,im);

	// Weergave van de rekentijd
	stop = clock();
	cout << "Computation time: " << (double)(stop - start)/(double)(CLOCKS_PER_SEC) << " seconds" << endl;

	return 0;
}
