#include "mathematica.h"

double sinc(double x) {
	// PI Normalized sinc function
	if (fabs(x) < ZERO) {
		return 1.0;
	} else {
		return sin(M_PI*x)/(M_PI*x);
	}
}

double hermite(double x,int order) {
	double result;

	switch (order) {
		case 0 :
			result = 1.0;
			break;
		case 1 :
			result = 2.0*x;
			break;
		default :
			result = 2.0*x*hermite(x,order - 1) - 2.0*double(order - 1)*hermite(x,order - 2);
			break;
	}

	return result;
}

long factorial(long x) {
	if (x == 0) {
		return 1;
	} else {
		return x*factorial(x - 1);
	}
}

doublevector gaussian_derivative(int size,int order,double scale) {
	if (size == 0) {
		int temp = (int)(((double)(order) + 1.0)*4.0*ceil(scale));
		if (temp > 1) {
			size = 2*temp + 1;
		} else {
			size = 3;
		}
	}

	doublevector result(size);
	doublevector norm(size);
	int boundary = size >> 1;
	double x;
	long fac = factorial((long)(order));
	double sum = 0.0;

	if (scale > ZERO) {
		for (int i = 0; i < size; i++) {
			x = (double)(i) - (double)(boundary);
			norm(i) = pow(x,(double)(order))/(double)(fac);
			result(i) = (-1.0/pow((scale*sqrt(2.0)),order))*hermite((x/(scale*sqrt(2.0))),order)*(1.0/(scale*sqrt(2.0*M_PI)))*exp(-x*x/(2.0*scale*scale));
			sum += norm(i)*result(i);
		}

		// Normalization
		for (int i = 0; i < size; i++) {
			result(i) /= sum;
		}
	} else {
		for (int i = 0; i < size; i++) {
			result(i) = 0.0;
		}
		result(boundary) = 1.0;
	}

	return result;
}

double gaussian(double x,double sigma) {
	double result = (1.0/(sigma*sqrt(2.0*M_PI)))*exp(-(x*x)/(2.0*sigma*sigma));

	return result;
}
