/*
Version: 0.99
Date: 04/08/05
*/

#ifndef MATRIX_H
#define MATRIX_H

#include <fstream>
#include <iostream>

using namespace std;
#include "defs.h"

template<class T> vector<T> sort(vector<T> v) {
	vector<T> u = v;
	vector<T> result(v.length);
	T minvalue,maxvalue;
	int pos = find_vector_max(v,maxvalue);

	result(v.length - 1) = maxvalue;
	maxvalue += (T)(1);

	for (int i = 0; i < (u.length - 1); i++) {
		pos = find_vector_min(u,minvalue);
		result(i) = minvalue;
		u(pos) = maxvalue;
	}

	return result;
}

template<class T> vector<T> get_row(matrix<T>& m,uint row) {
	vector<T> result(m.width);

	for (int i = 0; i < m.width; i++) {
		result(i) = m(row,i);
	}

	return result;
}

template<class T> vector<T> get_col(matrix<T>& m,uint col) {
	vector<T> result(m.height);

	for (int i = 0; i < m.height; i++) {
		result(i) = m(i,col);
	}

	return result;
}

template<class T> void set_row(matrix<T>& m,vector<T>& v,uint row) {
	for (int i = 0; i < m.width; i++) {
		m(row,i) = v(i);
	}
}

template<class T> void set_col(matrix<T>& m,vector<T>& v,uint col) {
	for (int i = 0; i < m.height; i++) {
		m(i,col) = v(i);
	}
}

template<class T> int find_vector_min(vector<T>& v,T& value) {
	int result = 0;
	value = v(0);

	for (int i = 1; i < v.length; i++) {
		if (v(i) < value) {
			value = v(i);
			result = i;
		}
	}

	return result;
}

template<class T> int find_vector_max(vector<T>& v,T& value) {
	int result = 0;
	value = v(0);

	for (int i = 1; i < v.length; i++) {
		if (v(i) > value) {
			value = v(i);
			result = i;
		}
	}

	return result;
}

template<class T> coordinate find_matrix_min(matrix<T>& m,T& value) {
	coordinate result;
	result.x = 0;
	result.y = 0;
	value = m(0,0);

	for (int i = 0; i < m.height; i++) {
		for (int j = 0; j < m.width; j++) {
			if (m(i,j) < value) {
				value = m(i,j);
				result.x = (double)(j);
				result.y = (double)(i);
			}
		}
	}

	return result;
}

template<class T> coordinate find_matrix_max(matrix<T>& m,T& value) {
	coordinate result;
	result.x = 0.0;
	result.y = 0.0;
	value = m(0,0);

	for (int i = 0; i < m.height; i++) {
		for (int j = 0; j < m.width; j++) {
			if (m(i,j) > value) {
				value = m(i,j);
				result.x = (double)(j);
				result.y = (double)(i);
			}
		}
	}

	return result;
}

template<class T> matrix<T> crop(matrix<T>& m,int y1,int x1,int y2,int x2) {
	matrix<T> result(y2 - y1 + 1,x2 - x1 + 1);

	for (int i = 0; i < result.height; i++) {
		for (int j = 0; j < result.width; j++) {
			result(i,j) = m(i + y1,j + x1);
		}
	}

	return result;
}

template<class T> double mean_matrix_value(matrix<T> m) {
	double result = 0.;

	for (int i = 0; i < m.height; i++) {
		for (int j = 0; j < m.width; j++) {
			result += m(i,j);
		}
	}

	result /= m.height*m.width;

	return result;
}

template<class T> double mean_vector_value(vector<T> v) {
	double result = 0.;

	for (int i = 0; i < v.length; i++) {
		result += v(i);
	}

	result /= v.length;

	return result;
}

template<class T> double variance_vector_value(vector<T> v,double mean) {
	double result = 0.;

	for (int i = 0; i < v.length; i++) {
		result += pow((v(i) - mean),2);
	}

	result /= (v.length - 1);

	return result;
}

template<class T> double median_vector_value(vector<T> v) {
	double result;

	vector<T> u = sort(v);

	if (u.length%2) {
		result = (double)(u(u.length >> 1));
	} else {
		result = ((double)(u((u.length >> 1) + 1)) + (double)(u(u.length >> 1)))/2.0;
	}

	return result;
}

template<class T> vector<T> convert_matrix2vector(matrix<T> m) {
	vector<T> result(m.height*m.width);
	int index = 0;

	for (int i = 0; i < m.height; i++) {
		for (int j = 0; j < m.width; j++) {
			result(index) = m(i,j);
			index++;
		}
	}

	return result;
}

template<class T> double median_matrix_value(matrix<T> m) {
	vector<T> v = convert_matrix2vector(m);

	double result = median_vector_value(v);

	return result;
}

template<class T> double sum_matrix_value(matrix<T> m) {
	double result = 0.;

	for (int i = 0; i < m.height; i++) {
		for (int j = 0; j < m.width; j++) {
			result += m(i,j);
		}
	}

	return result;
}

template<class T> double sum_vector_value(vector<T> v) {
	double result = 0.;

	for (int i = 0; i < v.length; i++) {
		result += v(i);
	}

	return result;
}

template<class T> void write_matrix(char* filename,matrix<T>& m) {
	fstream output;
	output.open(filename,ios::out);

	if(!output) {
		exit(1);
	}

	for (int i = 0; i < m.height; i++) {
		output << m(i,0);
        for (int j = 1; j < m.width; j++) {
			output << " " << m(i,j);
        }
		output << endl;
    }
}

template<class T> matrix<T> transpose(matrix<T>& m) {
	matrix<T> result(m.width,m.height);

	for (int i = 0; i < m.height; i++) {
		for (int j = 0; j < m.width; j++) {
			result(j,i) = m(i,j);
		}
	}

	return result;
}

template<class T> doublematrix inverse(matrix<T>& m) {
	doublematrix result(m.height,m.width);
	if (m.height != m.width) {
		cout << "Matrix must be square !" << endl;
		return result;
	}
	doublematrix tempmatrix(m.height,m.width << 1);
	int row = 0;
	double temp;

	for (int i = 0; i < result.height; i++) {
		for (int j = 0; j < result.width; j++) {
			result(i,j) = 0.;
		}
	}

	for (int i = 0; i < tempmatrix.height; i++) {
		for (int j = 0; j < m.width; j++) {
			tempmatrix(i,j) = (long double)(m(i,j));
            if (i == j) {
                tempmatrix(i,j + m.width) = 1.;
            } else {
				tempmatrix(i,j + m.width) = 0.;
            }
		}
	}

	for (int j = 0; j < m.width; j++) {
		if (tempmatrix(j,j) == 0.) {
			// Search non-zero element in column
			row = j;
			for (int i = j; i < tempmatrix.height; i++) {
				if (fabs(tempmatrix(i,j)) > 0.) {
					row = i;
				}
			}
			if (row == j) {
				cout << "There is no existing inverse matrix !" << endl;
				return result;
			} else {
				// SWAP row i and j
				for (int k = 0; k < tempmatrix.width; k++) {
					temp = tempmatrix(row,k);
					tempmatrix(row,k) = tempmatrix(j,k);
					tempmatrix(j,k) = temp;
				}
			}
		}

		// Row operations
		for (int i = 0; i < tempmatrix.height; i++) {
			temp = tempmatrix(i,j);
			if ((temp != 0.) && (i != j)) {
				for (int k = 0; k < tempmatrix.width; k++) {
					tempmatrix(i,k) = tempmatrix(i,k)*tempmatrix(j,j) - temp*tempmatrix(j,k);
				}
			}
		}

		// Rescale all rows
		for (int i = 0; i < tempmatrix.height; i++) {
			temp = tempmatrix(i,i);
			for (int j = 0; j < tempmatrix.width; j++) {
				tempmatrix(i,j) /= temp;
			}
		}
	}

	result = crop(tempmatrix,0,m.width,m.height - 1,(m.width << 1) - 1);

	return result;
}


template<class T> matrix<T> product(matrix<T>& m1,matrix<T>& m2) {
	matrix<T> result(m1.height,m2.width);
	if (m1.width != m2.height) {
		cout << "Inner dimensions must be equal !" << endl;
		return result;
	}

	for (int i = 0; i < result.height; i++) {
		for (int j = 0; j < result.width; j++) {
			result(i,j) = (T)(0);
			for (int k = 0; k < m1.width; k++) {
				result(i,j) += m1(i,k)*m2(k,j);
			}
		}
	}

	return result;
}

#endif // MATRIX_H
