#include "ppm.h"

frame image_saturation(frame& image) {
	frame result = image;
	
	for (int i = 0; i < image.width; i++) {
		for (int j = 0; j < image.height; j++) {
			if (result(j,i).red < 0.0) {
				result(j,i).red = 0.0;
			}
			if (result(j,i).red > 255.0) {
				result(j,i).red = 255.0;
			}
			if (result(j,i).green < 0.0) {
				result(j,i).green = 0.0;
			}
			if (result(j,i).green > 255.0) {
				result(j,i).green = 255.0;
			}
			if (result(j,i).blue < 0.0) {
				result(j,i).blue = 0.0;
			}
			if (result(j,i).blue > 255.0) {
				result(j,i).blue = 255.0;
			}
		}
	}
	return result;
}

frame read_ppm(char* filename) {
    FILE* input;
	char data[1024];
	byte buffer[3];

	if((input = fopen(filename,"rb")) == NULL) {
		exit(1);
	}
	
	// Magic code
	fgets(data,1024,input);
	if(data[0] != 'P' && data[1] != '6') {
		exit(1);
	}
	fgets(data,1024,input);
	while(data[0] == '#') {
		fgets(data,1024,input);
	}

	// Height & width
	int width = 0;
	int height = 0;
	sscanf(data,"%d %d",&width,&height);
	
	frame image(height,width);
	
	fgets(data,1024,input);
	while(data[0] == '#') {
		fgets(data,1024,input);
	}

	// Pixeldata
	for (int i = 0; i < image.height; i++) {
		for (int j = 0; j < image.width; j++) {
			fread(buffer,sizeof(byte),3,input);
			image(i,j).red = (double)(buffer[0]);
			image(i,j).green = (double)(buffer[1]);
			image(i,j).blue = (double)(buffer[2]);
		}
	}
	
	fclose(input);
	return image;
}

void write_ppm(char* filename,frame& image) {
	FILE* output;
	byte buffer[3];

	if((output = fopen(filename,"wb")) == NULL ) {
		exit(1);
	}

	fprintf(output,"P6\n%d %d\n255\n",image.width,image.height);

	frame imsat = image_saturation(image);

	for (int i = 0; i < image.height; i++) {
        for (int j = 0; j < image.width; j++) {
			buffer[0] = (byte)(ROUND(imsat(i,j).red));
			buffer[1] = (byte)(ROUND(imsat(i,j).green));
			buffer[2] = (byte)(ROUND(imsat(i,j).blue));
            fwrite(buffer,sizeof(byte),3,output);
        }
    }

	fclose(output);
}

ppmsequence::ppmsequence(char* filename) {
	name = filename;

	frames = 0;
	height = 0;
	width = 0;
	begin = 0;
}

ppmsequence::~ppmsequence() {
	frames = 0;
	height = 0;
	width = 0;
	begin = 0;
   
	name = NULL;
}

void ppmsequence::read_video() {
	FILE* input;
	char data[1024];

	if((input = fopen(name,"rt")) == NULL) {
		exit(1);
	}

	int end = 0;
	fgets(data,1024,input);
	sscanf(data,"%d %d %s",&begin,&end,fname);

	frames = end - begin + 1;

	frame image = get_frame(begin);
	height = image.height;
	width = image.width;

	fclose(input);
}

void ppmsequence::write_video() {
	FILE* output;

	if((output = fopen(name,"wt")) == NULL) {
		exit(1);
	}

	int end = begin + frames - 1;
	fprintf(output,"%d %d %s",begin,end,fname);

	fclose(output);
}

frame ppmsequence::get_frame(uint index) {
	char filename[255];
	sprintf(filename,fname,begin + index);

	frame image = read_ppm(filename);

	return image;
}

void ppmsequence::write_frame(uint index,frame& image) {
	char filename[255];
	sprintf(filename,fname,begin + index);

	write_ppm(filename,image);
}
